//#include <stdio.h>

#include "env.h"
#include "hash.h"

#define EMPTY_DATA -1

#define TIMEOUT_RECEIVE_SIGNAL 10
#define TIMEOUT_SHOW_MESSAGE 3
#define TIMEOUT_PRESS_KEY 10
#define TIMEOUT_CHOOSE_PAD 10

#define MSG_LISTEN_TIME_ELAPSED "Can't catch signal"
#define MSG_RECORD_TIME_ELAPSED "Nothing was pressed"

#define MSG_PRESS_KEY "Press key to bind"
#define MSG_LISTEN "Listen for signal"
#define MSG_KEY_BINDED "Key was binded"
#define MSG_CHOOSE_PAD "Choose pad"
#define MSG_PAD_NOT_CHOOSEN "Not choosen"
#define MSG_SLEEP "Sleep"

#define REPEAT_DELAY 300

enum Event
{
	e_start,       		// e_0
	e_key_pressed,		// e_1
	e_time_elapsed,		// e_2
	e_receive_signal	// e_3
};

enum State
{
	y_init,				// y_0
	y_sleep,			// y_1
	y_listen,			// y_2
	y_record,			// y_3
	y_show_msg,			// y_4
	y_choose_pad		// y_5
};

enum State state = y_init;   

void go(enum State new_state) 
{
	state = new_state;
}

void repeat_delay() 
{
	int i, j;
	for (i = 0; i < REPEAT_DELAY; i++)
		for (j = 0; j < REPEAT_DELAY; j++)
			;
}

void send(int key)   // z_1
{
	enum bool exists;
	int signal = hash_get(key, &exists);
	if (exists) {
		send_signal(signal);
		repeat_delay();
	}
}

void msg(char* msg) 
{
	go(y_show_msg);
	print(msg);
}

void some_init()
{
	hash_init();	
}


void automata (enum Event event, int _data)
{	
	enum State old_state = state;
	static int key_to_bind;				
	switch (state) {
		case y_init:			
			if (event == e_start) {
				some_init();  // z_0
				go(y_sleep);
			}				
			break;

		case y_sleep:			
			if (event == e_key_pressed)	{				

				if (_data == KEY_RECORD) { // C_0
					go(y_record);

				} else if (_data == KEY_CHANGE) { // C_1
					go(y_choose_pad);

				} else {
					send(_data);   // z_1
					go(y_sleep);
				}
			}
			break;

		case y_record:
			if (event == e_key_pressed) {
				if (_data != KEY_RECORD && _data != KEY_CHANGE) { // C_2
					key_to_bind = _data; // z_2
					go(y_listen);
				}

			} else if (event == e_time_elapsed) {
				msg(MSG_RECORD_TIME_ELAPSED);   // z_4	
			}
			break;

		case y_listen:
			if (event == e_receive_signal) {
				// z_3
				hash_bind(key_to_bind, _data); 
				msg(MSG_KEY_BINDED);		   

			} else if (event == e_time_elapsed) {
				msg(MSG_LISTEN_TIME_ELAPSED);  // z_5  	
			}
			break;

		case y_choose_pad:
			if (event == e_key_pressed && is_numeric_key(_data)) { // C_3							
				hash_change_pad(to_numeric(_data));				
				go(y_sleep);
			} else if (event == e_time_elapsed) {
				msg(MSG_PAD_NOT_CHOOSEN); //z_6	
			}
			break;

		case y_show_msg:
			if (event == e_time_elapsed) {
				go(y_sleep);
			}
			break;	
	}

	if (state == old_state) {
		return;
	}

	switch (state) {
		case y_sleep:
			print(MSG_SLEEP);	
			break;

		case y_listen:
			// z_8
			print(MSG_LISTEN); 
			set_timer(TIMEOUT_RECEIVE_SIGNAL); 
			break;

		case y_record:
			// z_9
			print(MSG_PRESS_KEY);
			set_timer(TIMEOUT_PRESS_KEY); 
			break;

		case y_show_msg:
			set_timer(TIMEOUT_SHOW_MESSAGE);   			// z_10
			break;

		case y_choose_pad:
			// z_11
			print(MSG_CHOOSE_PAD);			
			set_timer(TIMEOUT_CHOOSE_PAD);
			break;
	}
}

void start(void) 
{
	int key, signal;
	init_all();
	automata(e_start, EMPTY_DATA);
	while (true) {
		if (key_pressed(&key))
			automata(e_key_pressed, key);

		if (time_elapsed())
			automata(e_time_elapsed, EMPTY_DATA);

		if (receive_signal(&signal))
			automata(e_receive_signal, signal);
	}
}

void main(void)
{
	start();
}